@@ -55,6 +55,8 @@ class SubsequenceSortFilterProxyModel(QtCore.QSortFilterProxyModel):
 
     def filterAcceptsRow(self, row, _):
         completion = self.sourceModel().data(self.sourceModel().index(row, 0))
+        if completion is None or self.prefix is None:
+            return False
         if len(completion) < len(self.prefix):
             return False
         if len(self.prefix) == 1:
@@ -289,9 +291,12 @@ class CodeCompletionMode(Mode, QtCore.QObject):
         self._filter_mode = self.FILTER_FUZZY
         self._last_cursor_line = -1
         self._last_cursor_column = -1
+        self._last_completion_prefix = ''
         self._tooltips = {}
         self._show_tooltips = False
         self._request_id = self._last_request_id = 0
+        self._working = False
+        self._stylesheet_initialized = False
 
     def clone_settings(self, original):
         self.trigger_key = original.trigger_key
@@ -325,10 +330,10 @@ class CodeCompletionMode(Mode, QtCore.QObject):
         self._completer.highlighted.connect(self._display_completion_tooltip)
 
     def on_install(self, editor):
+        Mode.on_install(self, editor)
         self._create_completer()
         self._completer.setModel(QtGui.QStandardItemModel())
         self._helper = TextHelper(editor)
-        Mode.on_install(self, editor)
 
     def on_uninstall(self):
         Mode.on_uninstall(self)
@@ -406,7 +411,7 @@ class CodeCompletionMode(Mode, QtCore.QObject):
             if event.text() in self._trigger_symbols:
                 # symbol trigger, force request
                 self._reset_sync_data()
-                self.request_completion()
+                self.request_completion(triggered_by_symbol=True)
             elif len(word) >= self._trigger_len and event.text() not in \
                     self.editor.word_separators:
                 # Length trigger
@@ -451,6 +456,7 @@ class CodeCompletionMode(Mode, QtCore.QObject):
         self.editor.setTextCursor(cursor)
 
     def _on_results_available(self, results):
+        self._working = False
         debug("completion results (completions=%r), prefix=%s",
                         results, self.completion_prefix)
         context = results[0]
@@ -481,14 +487,21 @@ class CodeCompletionMode(Mode, QtCore.QObject):
         debug('reset sync data and hide popup')
         self._last_cursor_line = -1
         self._last_cursor_column = -1
+        self._last_completion_prefix = ''
         self._hide_popup()
 
-    def request_completion(self):
+    def request_completion(self, triggered_by_symbol=False):
+        if self._working:
+            return
+        self._working = True
         line = self._helper.current_line_nbr()
         column = self._helper.current_column_nbr() - \
             len(self.completion_prefix)
-        same_context = (line == self._last_cursor_line and
-                        column == self._last_cursor_column)
+        same_context = (
+            line == self._last_cursor_line and
+            column == self._last_cursor_column and
+            self.completion_prefix == self._last_completion_prefix
+        )
         if same_context:
             if self._request_id - 1 == self._last_request_id:
                 # context has not changed and the correct results can be
@@ -509,7 +522,8 @@ class CodeCompletionMode(Mode, QtCore.QObject):
                 'path': self.editor.file.path,
                 'encoding': self.editor.file.encoding,
                 'prefix': self.completion_prefix,
-                'request_id': self._request_id
+                'request_id': self._request_id,
+                'triggered_by_symbol': triggered_by_symbol
             }
             try:
                 self.editor.backend.send_request(
@@ -522,6 +536,7 @@ class CodeCompletionMode(Mode, QtCore.QObject):
                 debug('request sent: %r', data)
                 self._last_cursor_column = column
                 self._last_cursor_line = line
+                self._last_completion_prefix = self.completion_prefix
                 self._request_id += 1
                 return True
 
@@ -550,6 +565,7 @@ class CodeCompletionMode(Mode, QtCore.QObject):
             self._completer.popup().hide()
             self._last_cursor_column = -1
             self._last_cursor_line = -1
+            self._last_completion_prefix = ''
             QtWidgets.QToolTip.hideText()
 
     def _get_popup_rect(self):
@@ -572,7 +588,7 @@ class CodeCompletionMode(Mode, QtCore.QObject):
         :return:
         """
         
-        # If the cursor is not at the end of the documnet, then we check
+        # If the cursor is not at the end of the document, then we check
         # whether the next character is a word separator If not, then we don't
         # offer completions, because we don't want to complete in the middle of
         # a word.
@@ -605,6 +621,24 @@ class CodeCompletionMode(Mode, QtCore.QObject):
         if self._completer.widget() != self.editor:
             self._completer.setWidget(self.editor)
         self._completer.complete(self._get_popup_rect())
+        # The QCompleter popup doesn't respect the stylesheet. Here we
+        # reconstruct a basic stylesheet and directly apply it. There may be
+        # more elegant solutions, but this works.
+        if not self._stylesheet_initialized:
+            self._completer.popup().setStyleSheet(
+                '''
+                background: {};
+                color: {};
+                font-family: {};
+                font-size: {};
+                '''.format(
+                    self.editor.palette().base().color().name(),
+                    self.editor.palette().text().color().name(),
+                    self.editor.font_name,
+                    self.editor.font_size,
+                )
+            )
+            self._stylesheet_initialized = True
         self._completer.popup().setCurrentIndex(
              self._completer.completionModel().index(row, 0)
         )
